//
//  main.swift
//  alisma
//
//  Created by Howard Oakley on 12/03/2018.
//  Copyright © 2018 EHN & DIJ Oakley. All rights reserved.
//

import Foundation

enum OptionType: String {
    case toalias = "a"
    case topath = "p"
    case help = "h"
    case unknown
    
    init(value: String) {
        switch value {
        case "a": self = .toalias
        case "p": self = .topath
        case "h": self = .help
        default: self = .unknown
        }
    }
}

func getOption(_ option: String) -> (option: OptionType, value: String) {
    return (OptionType(value: option), option)
}

func printUsage() {
    print("alisma v1.1 usage:")
    print("alisma -a path file")
    print("to create an alias in file to the item at path")
    print("alisma -p alias")
    print("to return the current path to the file referenced by alias")
    print("alisma -h to show this usage information.")
}

func getPath(alias: String) -> String {
    var thePath = ""
    let aliasFileURL = URL.init(fileURLWithPath: alias)
    do {
        let theAliasURL = try URL.init(resolvingAliasFileAt: aliasFileURL)
        thePath = theAliasURL.path
        if (thePath == aliasFileURL.path) {
            thePath = "-1"
            fputs("Error: couldn't resolve alias.\n", stderr)
        }
    } catch {
        thePath = "-1"
        fputs("Error: couldn't find alias file.\n", stderr)
    }
    return thePath
}

func makeAlias(toPath: String, atPath: String) -> String {
    var thePath = ""
    let theAliasFURL = URL.init(fileURLWithPath: atPath)
    let theOrigURL = URL.init(fileURLWithPath: toPath)
    do {
        let theData = try theOrigURL.bookmarkData(options: [URL.BookmarkCreationOptions.suitableForBookmarkFile], includingResourceValuesForKeys: nil, relativeTo: nil)
        try URL.writeBookmarkData(theData, to: theAliasFURL)
        thePath = theAliasFURL.path
    } catch {
        thePath = "-1"
        fputs("Error: couldn't create new alias.\n", stderr)
    }
    return thePath
}

let argCount = CommandLine.argc
if (argCount > 2) {
    let first = CommandLine.arguments[1]
    let second = CommandLine.arguments[2]
    var third = ""
    if (argCount > 3) {
        third = CommandLine.arguments[3]
    }
    let indexStart = first.index(first.startIndex, offsetBy: 1)
    let theSubStr = first[indexStart...]
    let theSubStrStr = String(theSubStr)
    let (option, value) = getOption(theSubStrStr)
    switch option {
    case .toalias:
        let sixth = makeAlias(toPath: second, atPath: third)
        print(sixth)
    case .topath:
        let fifth = getPath(alias: second)
        print(fifth)
    case .help:
        printUsage()
    case .unknown:
        fputs("alisma v1.1: unknown option\n", stderr)
        printUsage()
    }
} else {
    printUsage()
}
